@@ -4,9 +4,9 @@ import sys
 from collections import defaultdict
 from unittest import TextTestRunner, TextTestResult as _TextTestResult
 
-from scrapy.command import ScrapyCommand
+from scrapy.commands import ScrapyCommand
 from scrapy.contracts import ContractsManager
-from scrapy.utils.misc import load_object
+from scrapy.utils.misc import load_object, set_environ
 from scrapy.utils.conf import build_component_list
 
 
@@ -58,35 +58,31 @@ class Command(ScrapyCommand):
 
     def run(self, args, opts):
         # load contracts
-        contracts = build_component_list(
-            self.settings['SPIDER_CONTRACTS_BASE'],
-            self.settings['SPIDER_CONTRACTS'],
-        )
-        conman = ContractsManager([load_object(c) for c in contracts])
+        contracts = build_component_list(self.settings.getwithbase('SPIDER_CONTRACTS'))
+        conman = ContractsManager(load_object(c) for c in contracts)
         runner = TextTestRunner(verbosity=2 if opts.verbose else 1)
         result = TextTestResult(runner.stream, runner.descriptions, runner.verbosity)
 
         # contract requests
         contract_reqs = defaultdict(list)
 
-        spman_cls = load_object(self.settings['SPIDER_MANAGER_CLASS'])
-        spiders = spman_cls.from_settings(self.settings)
+        spider_loader = self.crawler_process.spider_loader
 
-        for spider in args or spiders.list():
-            spider = spiders.create(spider)
-            requests = self.get_requests(spider, conman, result)
-            contract_reqs[spider.name] = []
+        with set_environ(SCRAPY_CHECK='true'):
+            for spidername in args or spider_loader.list():
+                spidercls = spider_loader.load(spidername)
+                spidercls.start_requests = lambda s: conman.from_spider(s, result)
 
-            if opts.list:
-                for req in requests:
-                    contract_reqs[spider.name].append(req.callback.__name__)
-            elif requests:
-                crawler = self.crawler_process.create_crawler(spider.name)
-                crawler.crawl(spider, requests)
+                tested_methods = conman.tested_methods_from_spidercls(spidercls)
+                if opts.list:
+                    for method in tested_methods:
+                        contract_reqs[spidercls.name].append(method)
+                elif tested_methods:
+                    self.crawler_process.crawl(spidercls)
 
         # start checks
         if opts.list:
-            for spider, methods in sorted(contract_reqs.iteritems()):
+            for spider, methods in sorted(contract_reqs.items()):
                 if not methods and not opts.verbose:
                     continue
                 print(spider)
@@ -101,15 +97,3 @@ class Command(ScrapyCommand):
             result.printSummary(start, stop)
             self.exitcode = int(not result.wasSuccessful())
 
-    def get_requests(self, spider, conman, result):
-        requests = []
-
-        for key, value in vars(type(spider)).items():
-            if callable(value) and value.__doc__:
-                bound_method = value.__get__(spider, type(spider))
-                request = conman.from_method(bound_method, result)
-
-                if request:
-                    requests.append(request)
-
-        return requests
